-- AGIL TIMEBOX - CONTACTOR-BOX
dofile('./interface/include.lua');
dofile('./interface/device.lua');

-- Information : Numéro de Version, Nom, Interface
function device.GetInformation()
	return { 
		version = 1.8, 
		code = 'agil_timebox',
		name = 'Agil TimeBox V2-V3', 
		class = 'chrono', 
		interface = { { type='serial_pulse', baudrate = '19200' } } 
	};
end

function device.OnConfiguration(node)
	local dlg = wnd.CreateDialog({
			icon = "./res/32x32_agil.png",
			label = "Configuration TimeBox",
			style = wndStyle.RESIZE_BORDER+wndStyle.CAPTION+wndStyle.CLOSE_BOX,
			width = 400,
			height = 300
	});
		
	dlg:LoadTemplateXML({ 
		xml = './device/agil_timebox.xml',
		node_name = 'root/panel',
		node_attr = 'name', 
		node_value = 'configuration'
	});
	
	local spinPulseWidth = dlg:GetWindowName('spin_pulse_width');
	spinPulseWidth:SetRange(0, 15000);
	spinPulseWidth:SetValue(node:GetAttribute('pulse_width', '10'));
	
	local spinDelayStart = dlg:GetWindowName('spin_delay_start');
	spinDelayStart:SetRange(0, 100);
	spinDelayStart:SetValue(node:GetAttribute('delay_start', '10'));

	local spinDelayInter1 = dlg:GetWindowName('spin_delay_inter1');
	spinDelayInter1:SetRange(0, 100);
	spinDelayInter1:SetValue(node:GetAttribute('delay_inter1', '4'));

	local spinDelayInter2 = dlg:GetWindowName('spin_delay_inter2');
	spinDelayInter2:SetRange(0, 100);
	spinDelayInter2:SetValue(node:GetAttribute('delay_inter2', '4'));

	local spinDelayFinish = dlg:GetWindowName('spin_delay_finish');
	spinDelayFinish:SetRange(0, 100);
	spinDelayFinish:SetValue(node:GetAttribute('delay_finish', '4'));

	local tb = dlg:GetWindowName('tb');
	local btnSave = tb:AddTool("Valider", "./res/32x32_save.png");
	tb:AddStretchableSpace();
	local btnRedirection = tb:AddTool("Redirection", "./res/32x32_clock.png");
	tb:AddStretchableSpace();
	local btnClose = tb:AddTool("Fermer", "./res/32x32_close.png");
	tb:Realize();

	dlg:Bind(eventType.MENU, function() device.OnSave(dlg, node); end, btnSave); 
	dlg:Bind(eventType.MENU, function() device.OnRedirection(dlg, node); end, btnRedirection); 
	dlg:Bind(eventType.MENU, function() dlg:EndModal(idButton.CANCEL); end, btnClose);

	dlg:Fit();
	dlg:ShowModal();
end	

function device.OnSave(dlg, node)
	if dlg ~= nil and node ~= nil then
		node:ChangeAttribute('pulse_width', dlg:GetWindowName('spin_pulse_width'):GetValue());
		
		node:ChangeAttribute('delay_start', dlg:GetWindowName('spin_delay_start'):GetValue());
		node:ChangeAttribute('delay_inter1', dlg:GetWindowName('spin_delay_inter1'):GetValue());
		node:ChangeAttribute('delay_inter2', dlg:GetWindowName('spin_delay_inter2'):GetValue());
		node:ChangeAttribute('delay_finish', dlg:GetWindowName('spin_delay_finish'):GetValue());

		app.GetXML():SaveFile();
		dlg:EndModal(idButton.OK);
	end
end

function device.OnRedirection(dlg, node)
	if dlg ~= nil and node ~= nil then
		wnd.DeviceChronoConfig(node, dlg);
	end
end

function device.OnInit(params, node)
	-- Force serial_pulse
	if type(params.type) == 'string' and params.type == 'serial' then
		params.type = 'serial_pulse';
	end
	
	device.node = node;
	if device.node ~= nil then
		local clockOffset = device.node:GetAttribute('clockOffset', '0');
		if clockOffset ~= '0' then
			device.clockOffset = tonumber(clockOffset);
			app.GetAuiMessage():AddLineWarning('TIMEBOX OFFSET = '..clockOffset);
		end
	end

	-- Appel OnInit Metatable
	mt_device.OnInit(params);

	-- Menu Chrono : Synchro Top
	wnd.DeviceAddMenuSynchroTop();

end

function device.SetChronoSynchroTop(clockTimeSynchro)
	device.clockTimeSynchro = clockTimeSynchro;
	device.clockModeSynchroTop = true;
end

function device.OnSerial(evt)
	if evt:GetInt() == serialNotify.CONNECTION then
		app.GetAuiMessage():AddLineSuccess(evt:GetString());
	
		if device.node ~= nil then
			serial.SetPulseWidth(mt_device.obj, device.node:GetAttribute('pulse_width'));
			serial.SetDelay(mt_device.obj, 0, device.node:GetAttribute('delay_start'));
			serial.SetDelay(mt_device.obj, 1, device.node:GetAttribute('delay_inter1'));
			serial.SetDelay(mt_device.obj, 2, device.node:GetAttribute('delay_inter2'));
			serial.SetDelay(mt_device.obj, -1, device.node:GetAttribute('delay_finish'));
		end
		
		local parentFrame = wnd.GetParentFrame();
		parentFrame:Bind(eventType.SERIAL_PULSE, device.OnSerialPulse, mt_device.obj);
		mt_device.obj:StartModePulse();
	elseif evt:GetInt() == serialNotify.LOST then
		app.GetAuiMessage():AddLineWarning(evt:GetString());
	else
		app.GetAuiMessage():AddLineError(evt:GetString());
	end
end

function device.OnSerialPulse(evt)
	device.clockOffset = device.clockOffset or 0;
	device.clockModeSynchroTop = device.clockModeSynchroTop or false;
	
	if device.clockModeSynchroTop == true then
		-- Synchro Top : Prise de l'offset ...
		local current = evt:GetExtraLong();
		device.clockTimeSynchro = device.clockTimeSynchro or current;
		device.clockOffset = device.clockTimeSynchro - current;
		device.clockModeSynchroTop = false;
		app.GetAuiMessage():AddLineSuccess("SYNCHRO TOP TIMEBOX OK : "..app.TimeToString(device.clockTimeSynchro, "%2hh%2m:%2s"));
	else
		-- Cas STD 
		local passage = evt:GetInt();
		if (passage & 8) ~= 0 then AddTimePassage(evt:GetExtraLong() + device.clockOffset, 0) end -- Depart
		if (passage & 2) ~= 0 then AddTimePassage(evt:GetExtraLong() + device.clockOffset, 1) end -- Inter 1
		if (passage & 4) ~= 0 then AddTimePassage(evt:GetExtraLong() + device.clockOffset, 2) end -- Inter 2
		if (passage & 1) ~= 0 then AddTimePassage(evt:GetExtraLong() + device.clockOffset, -1) end -- Arrivée
	end
end

function AddTimePassage(chrono, passage)
	app.SendNotify("<passage_add>", 
		{ time = chrono,  passage = passage, device = 'agil_timebox_v2' }
	);
end

function device.OnClose()
	if mt_device.obj ~= nil then 
		mt_device.obj:StopModePulse();
	end
	
	-- Appel OnClose Metatable
	mt_device.OnClose();

	-- SOS clockOffset 
	if device.node ~= nil then
		device.clockOffset = device.clockOffset or 0;
		if device.clockOffset == 0 then
			device.node:ChangeAttribute("clockOffset", '0');
		else
			device.node:ChangeAttribute("clockOffset", tostring(device.clockOffset));
		end
	end
	
end
