-- Matrix Led
dofile('./interface/interface.lua');
dofile('./interface/device.lua');
dofile('./interface/adv.lua');

-- Information : Numro de Version, Nom, Interface
function device.GetInformation(node)
	return { 
		version = 2.3, 
		code = 'led_display', 
		name = function() 
				local txt = 'Tableau Led';
				if node ~= nil then
					txt = txt..' ['..node:GetAttribute('col','64')..'x'..node:GetAttribute('row','16')..']'; 
				end
				return txt;
			end, 
		class = 'display', 
		interface = { 
				{ type='tcp', port = 7000, hostname = '192.168.74.141' },
				{ type='tcp_async', port = 7000, hostname = '192.168.74.141' },
				{ type='serial', baudrate = '1200,2400,4800,9600,19200,38400,57600' },
				{ type='none' } 
		}
	};
end	

ledDisplayTarget = 
{ 
	'agil_pi_led',
	''
}

ledDisplayMode = 
{ 
	{ code = 'live', label = 'En Course' },
	{ code = 'message', label = 'Hors Course' },
}

stateBib = {
	RUNNING			= 1,
	INTER			= 2,
	FINISH	 		= 3
};

function GetModeLabel(code)
	for i=1,#ledDisplayMode do
		if ledDisplayMode[i].code == code then
			return ledDisplayMode[i].label;
		end
	end
	return '?';
end

function GetScenarioLabel(code)
	for i=1,#ledDisplayTemplate do
		if ledDisplayTemplate[i].code == code then
			return ledDisplayTemplate[i].label;
		end
	end
	return '?';
end

function LoadScenario(scenarioLua)
	scenarioPath = './device/led_display_res/'..scenarioLua;
	local fn = loadfile(scenarioPath);
	if type(fn) == 'function' then
		fn();
		return true;
	else
		adv.Error('Chargement Impossible du Scnario '..scenarioPath);
		return false;
	end
end

function PlayScenario(code)
	if type(ledDisplayTemplate) == 'table' then
		for i=1,#ledDisplayTemplate do
			if ledDisplayTemplate[i].code == code then
				ledDisplayTemplate[i].fn();
				return;
			end
		end
	end
end

-- OnContextMenu
function OnContextMenu(evt)
	local menuContext =  menu.Create();

	local btnConfig = menuContext:Append({label="Configuration", image ="./res/16x16_configure.png"});
	led_panel:Bind(eventType.MENU, function() device.OnConfiguration(device.led_node) end, btnConfig);
	local btnMessage = menuContext:Append({label="Message", image ="./res/16x16_edit.png"});
	led_panel:Bind(eventType.MENU, function() device.OnMessage(device.led_node) end, btnMessage);

	if device.led_mode == 'live' then
		local btnModeLiveOff = menuContext:Append({label='Basculement Mode '..GetModeLabel('message'), image ="./res/16x16_organize.png"});
		led_panel:Bind(eventType.MENU, function() device.led_mode = 'message'; ChangeMode(); end, btnModeLiveOff);
	elseif device.led_mode == 'message' then
		local btnModeLiveOn = menuContext:Append({label='Basculement Mode '..GetModeLabel('live'), image ="./res/16x16_organize.png"});
		led_panel:Bind(eventType.MENU, function() device.led_mode = 'live'; ChangeMode(); end, btnModeLiveOn);
	end
	
	menuContext:AppendSeparator();
	local btnClock = menuContext:Append({label="Test Horloge", image ="./res/16x16_clock.png"});
	led_panel:Bind(eventType.MENU, LedDisplay_Test_Clock, btnClock);
	local btnSound = menuContext:Append({label="Test Son", image ="./res/16x16_notification.png"});
	led_panel:Bind(eventType.MENU, LedDisplay_Test_Sound, btnSound);
	local btnFinish = menuContext:Append({label="Test Finish", image ="./res/16x16_house.png"});
	led_panel:Bind(eventType.MENU, LedDisplay_Test_Finish, btnFinish);

	led_panel:PopupMenu(menuContext);
	menuContext:Delete();
end

function ChangeMode(ledMode)
	-- Effacement 
	SendLedCommands('-c all -s');
	
	if device.led_mode == 'message' then
		LedDisplay_Message();
	end

	ChangeCaption();
end

function ChangeCaption()
	local mgr = app.GetAuiManager();
	
	if mgr ~= nil and led_panel ~= nil then
		local caption = "LED-DISPLAY ("..
			tostring(device.led_row)..'x'..
			tostring(device.led_col)..') '..
			GetScenarioLabel(device.led_scenario)..' / '.. 
			GetModeLabel(device.led_mode);

		mgr:SetCaption(led_panel, caption);
		mgr:Update();
	end
end

-- Configuration
function device.OnConfiguration(node)
	if wnd.LedDisplayConfig(node) == idButton.OK then
		if type(device.led_params) == 'table' then
			-- Reload ...
			device.OnClose();
			device.OnInit(device.led_params, device.led_node);
		end
	end
end

-- Message
function device.OnMessage(node)
	if wnd.LedDisplayMessage(node) == idButton.OK then
		if type(device.led_params) == 'table' then
			-- Reload ...
			device.OnClose();
			device.led_node:ChangeAttribute('mode', device.led_mode);
			device.OnInit(device.led_params, device.led_node);
		end
	end
end

-- Event Socket Client
function device.OnSocketClient(evt)
	-- Appel OnSocketClient Metatable
	mt_device.OnSocketClient(evt);
	
	if evt:GetSocketEvent() == socketNotify.CONNECTION then
		-- Lightness 
		SendLedCommands('-lightness '..tostring(device.led_lightness));

		if device.led_mode == 'message' then
			LedDisplay_Message();
		end
	end
end

-- Ouverture
function device.OnInit(params, node)

	-- Chargement du Scenario (template)
	if LoadScenario(node:GetAttribute('template_file')) == false then
		return;
	end

	-- Appel OnInit Metatable
	mt_device.OnInit(params);

	-- Initialisation des Temps Tournant ...
	ClearBibRunning();

	-- Rcupration des infos de la course
	local rc, raceInfo = app.SendNotify('<race_load>');
	assert(rc);
	device.raceInfo = raceInfo;

	-- Prise Manche (Ski) ou Course et Phase (Canoe) 
	if device.raceInfo ~= nil then
		device.Code_course = tonumber(raceInfo.Code_course) or -1;
		device.Code_phase = tonumber(raceInfo.Code_phase) or -1;
		device.Code_manche = tonumber(raceInfo.Code_manche) or 1;
	else
		device.Code_course = -1;
		device.Code_phase = -1;
		device.Code_manche = 1;
	end

	-- Dfinition clef de la phase ou de la manche en cours ...
	device.key_current = '';
	if device.Code_course >= 1 and device.Code_phase >= 1 then
		-- Canoe
		device.key_current = tostring(device.Code_course)..'_'..tostring(device.Code_phase);
	else 
		-- Ski
		if device.Code_manche == 1 then
			device.key_current = '1';
		end
	end

	-- Discipline, Entite, Penalit ...
	device.discipline = '';
	device.entite = '';
	device.nb_penalty = tonumber(raceInfo.Nb_porte) or 0;
	
	local tEpreuve = device.raceInfo.tables.Epreuve;
	if tEpreuve ~= nil then
		-- Ski
		device.discipline = tEpreuve:GetCell('Code_discipline',0);	-- SL, GS, ...
		device.entite = tEpreuve:GetCell('Code_entite',0);	-- ESF, FFS, FIS
	else
		local tCompetition = device.raceInfo.tables.Competition;
		if tCompetition ~= nil then
			-- Canoe 
			device.discipline = tCompetition:GetCell('Code_activite',0);	-- DES, SLA, ...
			device.entite = 'FFCK';
		end
	end
	
	-- Creation Simulateur 
	led_panel = wnd.CreatePanel({ parent = app.GetAuiFrame() });
	led_panel:LoadTemplateXML({ 
		xml = './device/led_display.xml',
		node_name = 'root/panel',
		node_attr = 'name',
		node_value = 'led_simulator'
	});

	device.led_params = params;
	device.led_node = node;

	device.led_target = node:GetAttribute("target");
	device.led_mode = node:GetAttribute("mode");
	
	device.led_scenario = node:GetAttribute("template_code");
	device.delay_finish = tonumber(node:GetAttribute("delay_finish")) or 5000;
	device.delay_inter = tonumber(node:GetAttribute("delay_inter")) or 3000;
	
	device.led_row = tonumber(node:GetAttribute("row")) or 16;
	device.led_col = tonumber(node:GetAttribute("col")) or 96;
	device.led_lightness = tonumber(node:GetAttribute("lightness")) or 100;

	-- Initialisation Simulator ...
	led_simulator = led_panel:GetWindowName('led_simulator');
	led_simulator:LedSetMatrixSize(device.led_row, device.led_col);

	local sizePixel = tonumber(node:GetAttribute("simulator_pixel")) or 3;
	local sizePixelSpacing = tonumber(node:GetAttribute("simulator_pixel_spacing")) or 1;
	led_simulator:LedSetSize(sizePixel, sizePixelSpacing);
	
	local levelMessage = tonumber(node:GetAttribute("simulator_level_message")) or 0;
	led_simulator:LedSetLevelMessage(levelMessage);

	-- Caption
	local caption = "LED-DISPLAY ("..
		tostring(device.led_row)..'x'..
		tostring(device.led_col)..') '..
		GetScenarioLabel(device.led_scenario)..' / '.. 
		GetModeLabel(device.led_mode)
	;

	-- Affichage ...
	led_panel:Show(true);
	led_panel:Bind(eventType.CONTEXT_MENU, OnContextMenu, -1);
	
	local mgr = app.GetAuiManager();
	mgr:AddPane(led_panel, {
		icon = './res/16x16_agil.png',
		caption = caption,
		caption_visible = true,
		close_button = false,
		pin_button = true,
		show = true,

		float = true, 
		floating_position = {app.GetAuiFrame():GetDisplayArea().x+640, 1},
		floating_size = {405, 110},
		dockable = false
		
	});
	mgr:Update();

	-- Notify 
	app.BindNotify("<on_course>", OnNotifyOnCourse);
	app.BindNotify("<passage_insert>", OnNotifyPassageInsert);
	app.BindNotify("<bib_time>", OnNotifyBibTime);
	app.BindNotify("<forerunner_time>", OnNotifyForerunnerTimeBibTime);
	app.BindNotify("<penalty_add>", OnNotifyPenaltyAdd);
	app.BindNotify("<run_erase>", OnNotifyRunErase);
	
	device.state = stateBib.RUNNING;
	
	-- Lancement du Scenario (template)
	PlayScenario(device.led_scenario);

	-- Timer
	local parentFrame = app.GetAuiFrame();
	device.led_running_time =  tonumber(node:GetAttribute("running_time")) or 0;
	
	if device.led_running_time > 0 then
		timerRacer = timer.Create(parentFrame);
		timerRacer:Start(device.led_running_time);
		parentFrame:Bind(eventType.TIMER, OnTimer, timerRacer);
	end
	
	if device.led_mode == 'message' and mt_device.type ~= 'tcp' then
		LedDisplay_Message();
	end
	
end

function ClearBibRunning()
	tRunning = tRunning or {};

	tRunning.bib = '';
	tRunning.time = -1;
	tRunning.bibLoad = nil;
end

function GetIdentity(tRanking, row)
	col = tRanking:GetIndexColumn('Bateau');
	if col >= 0 then
		return tRanking:GetCell(col, row);
	else
		return tRanking:GetCell('Nom', row)..' '..tRanking:GetCell('Prenom', row):sub(1,1):upper()..'.';
	end
end

function SynchroTemplate(tpl)
	assert(type(tpl) == 'table');
	assert(type(tpl.fields) == 'table');
	assert(type(tpl.data) == 'table');

	local commands = '';
	
	for i=1,#tpl.fields do
		local field = tpl.fields[i];
		commands = commands..' '..GetLedCommands(field, tpl);
	end
	
	if commands ~= '' then
		commands = commands..' -s';
		SendLedCommands(commands);
	end
end

function GetTemplateFieldData(field, tpl)
	if type(field.data) == 'function' then
		return field.data(field, tpl);
	elseif type(field.data) == 'string' then
		if string.sub(field.data,1,1) == '"' and string.sub(field.data,-1) == '"' then
			return string.sub(field.data,2,string.len(field.data)-1);
		elseif tpl.data[field.data] ~= nil then
			return tpl.data[field.data];
		end
	end

	return '';
end

function GetTemplateKeyString(key, field, tpl, defaultValue)
	if type(key) == 'function' then
		return key(field, tpl);
	elseif type(key) == 'string' then
		return key;
	elseif key == nil then
		if defaultValue ~= nil then
			return defaultValue;
		else
			return '';
		end
	else
		return tostring(key);
	end
end

function GetTemplateKeyInteger(key, field, tpl, defaultValue)
	if type(key) == 'function' then
		return key(field, tpl);
	elseif type(key) == 'number' then
		return key;
	elseif key == nil then
		if defaultValue ~= nil then
			return defaultValue;
		else
			return 0;
		end
	else
		return tonumber(key);
	end
end

function GetTemplateKeyColor(key, field, tpl, defaultValue)
	
	if type(key) == 'function' then
		return key(field, tpl);
	elseif type(key) == 'string' then
		return key;
	elseif type(key) == 'userdata' then
		return key:GetRGB();
	elseif key ~= nil then
		return tostring(key);
	end
	
	if defaultValue ~= nil then
		return defaultValue;
	else
		return '0';
	end
end

function SendLedCommands(commands)
	if led_simulator ~= nil then
		led_simulator:LedCommands(commands);
	end
		
	if mt_device.type == 'tcp' then
		-- tcp
		socketClient.WriteString(mt_device.obj, commands..string.char(13));
	elseif mt_device.type == 'tcp_async' and type(mt_device.obj) == 'table' then
		-- tcp_async
		socket.SendTcpAsync(led_panel, mt_device.obj.hostname, mt_device.obj.port, commands..string.char(13));
	end
end

function GetLedCommandText(field, tpl)
	local txt = GetTemplateFieldData(field, tpl) or '';
	if string.len(txt) > 0 then
		local font = GetTemplateKeyString(field.font, field, tpl, 't8x8');
		local color = GetTemplateKeyColor(field.color, field, tpl, 'black');
		local x = GetTemplateKeyInteger(field.x, field, tpl, 0);
		local y = GetTemplateKeyInteger(field.y, field, tpl, 0);
		local optimize = GetTemplateKeyInteger(field.optimize, field, tpl, 1);
		local clear = GetTemplateKeyInteger(field.clear, field, tpl, 0);
		local align = GetTemplateKeyString(field.align, field, tpl, 'L');
		if align == 'L' then
			return '-'..font..' "'..txt..'" '..color..' '..x..' '..y..' '..optimize..' '..clear;
		else
			local w = GetTemplateKeyInteger(field.w, field, tpl, 0);
			return '-'..font..' "'..txt..'" '..color..' '..x..' '..y..' '..optimize..' '..clear..' '..align..' '..w;
		end
	else
		return '';
	end
end

function GetLedCommandScrollText(field, tpl)
	local txt = GetTemplateFieldData(field, tpl) or '';
	if string.len(txt) > 0 then
		local scrollfont = 'scroll'..GetTemplateKeyString(field.font, field, tpl, 't8x8'):sub(2);
		local color = GetTemplateKeyColor(field.color, field, tpl, 'black');
		local x = GetTemplateKeyInteger(field.x, field, tpl, 0);
		local y = GetTemplateKeyInteger(field.y, field, tpl, 0);
		local w = GetTemplateKeyInteger(field.w, field, tpl, 0);
		local h = GetTemplateKeyInteger(field.h, field, tpl, 0);
		local milliseconds = GetTemplateKeyInteger(field.milliseconds, field, tpl, 0);
		return '-'..scrollfont..' "'..txt..'" '..color..' '..x..' '..y..' '..w..' '..h..' '..milliseconds;
	else
		return '';
	end
end

function GetLedCommandLogo(field, tpl)
	local filename = GetTemplateKeyString(field.filename, field, tpl);
	local x = GetTemplateKeyInteger(field.x, field, tpl, 0);
	local y = GetTemplateKeyInteger(field.y, field, tpl, 0);
	return '-logo "'..filename..'" '..x..' '..y;
end

function GetLedCommandPixel(field, tpl)
	local color = GetTemplateKeyColor(field.color, field, tpl, 'black');
	local x = GetTemplateKeyInteger(field.x, field, tpl, 0);
	local y = GetTemplateKeyInteger(field.y, field, tpl, 0);
	return '-px '..color..' '..x..' '..y;
end

function GetLedCommandLine(field, tpl)
	local color = GetTemplateKeyColor(field.color, field, tpl, 'black');
	local x = GetTemplateKeyInteger(field.x, field, tpl, 0);
	local y = GetTemplateKeyInteger(field.y, field, tpl, 0);
	local w = GetTemplateKeyInteger(field.w, field, tpl, 0);
	local h = GetTemplateKeyInteger(field.h, field, tpl, 0);
	return '-ln '..color..' '..x..' '..y..' '..w..' '..h;
end

function GetLedCommandRectangle(field, tpl)
	local color = GetTemplateKeyColor(field.color, field, tpl, 'black');
	local x = GetTemplateKeyInteger(field.x, field, tpl, 0);
	local y = GetTemplateKeyInteger(field.y, field, tpl, 0);
	local w = GetTemplateKeyInteger(field.w, field, tpl, 0);
	local h = GetTemplateKeyInteger(field.h, field, tpl, 0);
	return '-rect '..color..' '..x..' '..y..' '..w..' '..h;
end

function GetLedCommandAnimation(field, tpl)
	local name = GetTemplateKeyString(field.name, field, tpl);
	local milliseconds = GetTemplateKeyInteger(field.milliseconds, field, tpl, 50);
	local x = GetTemplateKeyInteger(field.x, field, tpl, 0);
	local y = GetTemplateKeyInteger(field.y, field, tpl, 0);
	local loopback = GetTemplateKeyString(field.loopback, field, tpl);
	return '-animation play '..name..' '..milliseconds..' '..x..' '..y..' '..loopback;
end

function GetLedCommandPenaltyRect(field, tpl)
	local tabPena = GetTemplateFieldData(field, tpl);
	local nb_pena = tonumber(device.nb_penalty) or 0;
	local cmd = '';

	if nb_pena > 0 and type(tabPena) == 'table' and #tabPena == nb_pena then
		local x = GetTemplateKeyInteger(field.x, field, tpl, 0);
		local y = GetTemplateKeyInteger(field.y, field, tpl, 0);
		local w = GetTemplateKeyInteger(field.w, field, tpl, 1);
		local h = GetTemplateKeyInteger(field.h, field, tpl, 1);

		for i=1,nb_pena do
			local color = 'blue';
			if tabPena[i] == '0' then
				color = 'green';
			elseif tabPena[i] == '2' then
				color = 'yellow';
			elseif tabPena[i] == '50' then
				color = 'red';
			end
				
			cmd = cmd..' -rect '..color..' '..tostring(x+i-1)..' '..y..' '..w..' '..h;
		end
	end
	return cmd;
end

function GetLedCommandSound(field, tpl)
	return '-sound ';
end

function GetLedCommandClear(field, tpl)
	local option = GetTemplateKeyString(field.option, field, tpl);
	return '-c '..option;
end

function GetLedCommands(field, tpl)
	local cmd = '';
	if type(field) == 'table' then
		field.parent = tpl;
		if GetLedCondition(field, tpl, true) == true then
			local command = GetTemplateKeyString(field.command, field, tpl);
			if command == 'text' then
				cmd = cmd..' '..GetLedCommandText(field, tpl)
			elseif command == 'scrolltext' then
				cmd = cmd..' '..GetLedCommandScrollText(field, tpl)
			elseif command == 'logo' then
				cmd = cmd..' '..GetLedCommandLogo(field, tpl)
			elseif command == 'pixel' then
				cmd = cmd..' '..GetLedCommandPixel(field, tpl)
			elseif command == 'line' then
				cmd = cmd..' '..GetLedCommandLine(field, tpl)
			elseif command == 'rectangle' then
				cmd = cmd..' '..GetLedCommandRectangle(field, tpl)
			elseif command == 'animation' then
				cmd = cmd..' '..GetLedCommandAnimation(field, tpl)
			elseif command == 'sound' then
				cmd = cmd..' '..GetLedCommandSound(field, tpl)
			elseif command == 'penalty_rect' then
				cmd = cmd..' '..GetLedCommandPenaltyRect(field, tpl)
			elseif command == 'clear' then
				cmd = cmd..' '..GetLedCommandClear(field, tpl)
			else
				cmd = cmd..' -cmd???';
			end
		end
	end
	return cmd;
end

function GetLedCondition(field, tpl)
	local cond = GetTemplateKeyInteger(field.condition, field, tpl, 1);
	if cond == 1 or cond == true then
		return true;
	else
		return false;
	end
end

function OnNotifyRunErase(key, params)
	ClearBibRunning();
	if device.led_mode == 'live' then
		SendLedCommands('-c -s');
	end
end

function OnNotifyOnCourse(key, params)
	if type(params) == 'table' then 
	
		local t = nil;
		local need_delete = false;

		if type(params.table) == 'userdata' and app.GetNameSpace(params.table) == 'sqlTableGC' then
			t = params.table;
			need_delete = false;
		else
			t = sqlTable.ExtractTableUnique(params, 'table');
			need_delete = true;
		end
		
		if type(t) == 'userdata' then
			if t:GetNbRows() > 0 then
				tRunning.bib = t:GetCell('bib', t:GetNbRows()-1);
				tRunning.time = t:GetCellInt('time', t:GetNbRows()-1);

				if type(tRunning.bibLoad) ~= 'userdata' or tRunning.bibLoad:GetCell('Dossard', 0) ~= tRunning.bib then
					tRunning.bibLoad = GetBibLoad(tRunning.bib);
				end
			end
			if need_delete == true then
				t:Delete();
			end
		end
	end
end

function OnNotifyPassageInsert(key, params)
	showDNS = showDNS or false;
	if showDNS then
		local hour = tonumber(params.time);
		local bib = params.bib;
		local id = tonumber(params.passage);
		if id == 0 and hour == chrono.DNS and string.len(bib) > 0 then
			app.GetAuiMessage():AddLine('ABS ! Dossard '..bib);
			LedDisplay_Finish(bib);
		end
	end
end

function OnNotifyBibTime(key, params)

	if device.led_mode == 'live' then
		local bib = tonumber(params.bib) or 0;
		if bib <= 0 then
			return
		end

		local time_chrono = params.time_chrono;
		if time_chrono < 0 and time_chrono ~= chrono.DNF and time_chrono ~= chrono.DSQ then
			return
		end

		local idPassage = tonumber(params.passage) or -99;
		if idPassage == -1 then
			-- Arrive ...
			device.finish_data = {};
			device.finish_data.bib = bib;
			device.finish_data.time_chrono = time_chrono;
			
			if params.diff ~= nil then
				device.finish_data.diff_scratch_chrono = params.diff;
				device.finish_data.diff_scratch = params.diff;
			end
			
			if params.diff_categ ~= nil then
				device.finish_data.diff_chrono = params.diff_categ;
				device.finish_data.diff = params.diff_categ;
			end
			
			-- Dossard ok et Chrono Ok ...
			tickcount_finish = app.GetTickCount(); 
			device.state = stateBib.FINISH;
			LedDisplay_Finish(bib);
			
		elseif idPassage >= 1 then
			-- Inter1, Inter 2 ...
			if tRunning.bib == params.bib and device.state ~= stateBib.FINISH then
				-- Uniquement pour le coureur en course 
				device.inter_data = {};
				device.inter_data.bib = bib;
				device.inter_data.time_chrono = time_chrono;
				
				if params.diff ~= nil then
					device.inter_data.diff_scratch_chrono = params.diff;
					device.inter_data.diff_scratch = params.diff;
				end
			
				if params.diff_categ ~= nil then
					device.inter_data.diff_chrono = params.diff_categ;
					device.inter_data.diff = params.diff_categ;
				end

				-- Dossard ok et Chrono Ok ...
				tickcount_inter = app.GetTickCount(); 
				device.state = stateBib.INTER;
				LedDisplay_Inter(bib, idPassage);
			end
		end
	end
end

function OnNotifyForerunnerTimeBibTime(key, params)
	if device.led_mode == 'live' then
		local idPassage = tonumber(params.passage) or -99;
		finish_time = params.time or 0;

		if idPassage == -1 and device.entite == 'ESF' and finish_time > 0 then
			-- Arrive Ouvreur ...
			tickcount_finish = app.GetTickCount(); 
			device.state = stateBib.FINISH;
		end
	end
end

function ChangePenalty(bibTable, gate, penalty)
	if type(bibTable) == 'userdata' and bibTable:GetNbRows() > 0 then
		if gate >= 1 and gate <= device.nb_penalty then
			if penalty == 0 then
				bibTable:SetCell('@PENA_'..tostring(gate)..'_1',0, '0');
			elseif penalty == 2 then
				bibTable:SetCell('@PENA_'..tostring(gate)..'_1',0, '2');
			elseif penalty == 5 or penalty == 50 then
				bibTable:SetCell('@PENA_'..tostring(gate)..'_1',0, '50');
			else
				bibTable:SetCell('@PENA_'..tostring(gate)..'_1',0, ' ');
			end
		end
	end
end

function OnNotifyPenaltyAdd(key, params)
	if device.led_mode == 'live' then
		-- Mode en Course 
		local bib = tonumber(params.bib) or 0;
		local gate = tonumber(params.gate) or 0;
		local penalty = tonumber(params.penalty) or -1;
	--	adv.Alert('OnNotifyPenaltyAdd : bib='..tostring(params.bib)..', gate='..tostring(gate)..',penalty='..tostring(penalty));
		
		if device.state == stateBib.RUNNING then
			if bib == tonumber(tRunning.bib) then
				-- Pnalit sur le bateau en course
				ChangePenalty(tRunning.bibLoad, gate, penalty);
			end
		elseif device.state == stateBib.FINISH then
			if device.finish_data ~= nil and bib == tonumber(device.finish_data.bib) then
				-- Pnalit sur le bateau  l'arrive
				tickcount_finish = app.GetTickCount(); 
				LedDisplay_Finish(bib);
			end
		end
	end
end

-- Fermeture
function device.OnClose()
	-- Fermeture Timer
	if timerRacer ~= nil then 
		timerRacer:Delete() 
	end

	SendLedCommands('-c all -s');

	if led_panel ~= nil then
		local mgr = app.GetAuiManager();
		mgr:DeletePane(led_panel);
	end

	-- Appel OnClose Metatable
	mt_device.OnClose();
end

function OnTimer(evt)

	if device.led_mode == 'live' then
		-- Mode en Course ...
		if device.state == stateBib.RUNNING then
			LedDisplay_Running();
		elseif device.state == stateBib.INTER then
			local tickCount = app.GetTickCount();
			if tickCount - tickcount_inter >= device.delay_inter then
				device.state = stateBib.RUNNING;
				ClearBibRunning(); -- Remise  zro du dossard en cours 
				tickcount_inter = 0;
				SendLedCommands("-c -s");
			end
		elseif device.state == stateBib.FINISH then
			local tickCount = app.GetTickCount();
			if tickCount - tickcount_finish >= device.delay_finish then
				if ledTemplateFinish2 ~= nil and ledTemplateFinishLast == ledTemplateFinish then
					LedDisplay_Finish(device.finish_data.bib);
				else
					device.state = stateBib.RUNNING;
					ClearBibRunning(); -- Remise  zro du dossard en cours 
					tickcount_finish = 0;
					SendLedCommands("-c -s");
				end
			end
		end
	end
end

-- Chargement des informations lies au dossard
function GetBibLoad(bib)
--	adv.Alert('GetBibLoad:'..tostring(bib));
	local rc, bibLoad = app.SendNotify("<bib_load>", { bib = bib });
	if rc and type(bibLoad) == 'table' then
		return bibLoad.ranking;
	else
		return nil;
	end
end

function GetColumnTimeChrono(bibRanking)
	local col = bibRanking:GetIndexColumn('Tps_chrono'..device.key_current);
	if col >= 0 then
		return col;
	end
	
	col = bibRanking:GetIndexColumn('@CHRONO_'..device.key_current..'_1');
	if col >= 0 then
		return col;
	end

	return -1;
end

function GetColumnRkChrono(bibRanking)
	local col = bibRanking:GetIndexColumn('Cltc_chrono'..device.key_current);
	if col >= 0 then
		return col;
	end
	
	col = bibRanking:GetIndexColumn('@RK_CHRONO_'..device.key_current..'_1');
	if col >= 0 then
		return col;
	end

	return -1;
end

function GetColumnRkScratchChrono(bibRanking)
	local col = bibRanking:GetIndexColumn('Clt_chrono'..device.key_current);
	if col >= 0 then
		return col;
	end
	
	col = bibRanking:GetIndexColumn('@RK_CHRONO_SCRATCH_'..device.key_current..'_1');
	if col >= 0 then
		return col;
	end

	return -1;
end

function GetColumnTime(bibRanking)
	local col = bibRanking:GetIndexColumn('Tps'..device.key_current);
	if col >= 0 then
		return col;
	end
	
	col = bibRanking:GetIndexColumn('@TIME_'..device.key_current..'_1');
	if col >= 0 then
		return col;
	end

	return -1;
end

function GetColumnRk(bibRanking)
	local col = bibRanking:GetIndexColumn('Cltc'..device.key_current);
	if col >= 0 then
		return col;
	end
	
	col = bibRanking:GetIndexColumn('@RK_'..device.key_current..'_1');
	if col >= 0 then
		return col;
	end

	return -1;
end

function GetColumnRkScratch(bibRanking)
	local col = bibRanking:GetIndexColumn('Clt'..device.key_current);
	if col >= 0 then
		return col;
	end
	
	col = bibRanking:GetIndexColumn('@RK_SCRATCH_'..device.key_current..'_1');
	if col >= 0 then
		return col;
	end

	return -1;
end

function GetColumnTimeTotal(bibRanking)
	local col = bibRanking:GetIndexColumn('Tps');
	if col >= 0 then
		return col;
	end

	return -1;
end

function GetColumnRkTotal(bibRanking)
	local col = bibRanking:GetIndexColumn('Cltc');
	if col >= 0 then
		return col;
	end

	return -1;
end

function GetColumnRkScratchTotal(bibRanking)
	local col = bibRanking:GetIndexColumn('Clt');
	if col >= 0 then
		return col;
	end

	return -1;
end

function GetStringTime(time_ms, fmtShort, fmtLong, time_ms_scale)
	if time_ms > 0 and time_ms < time_ms_scale then
		return app.TimeToString(time_ms, fmtShort);
	else
		return app.TimeToString(time_ms, fmtLong);
	end
end

function SetTemplateDataPenalty(tpl, bibTable)
	if device.nb_penalty > 0 and tpl ~= nil and bibTable ~= nil then
		tpl.data.penalty = {};
		local penalty_total = 0;
		local penalty_count = 0;
		
		for p=1,device.nb_penalty do
			local pena = bibTable:GetCell('@PENA_'..tostring(p)..'_1',0);
			if pena == '0' or pena == '2' or pena == '50' then
				penalty_count = penalty_count + 1;
				penalty_total = penalty_total + tonumber(pena);
			end
			table.insert(tpl.data.penalty, pena);
		end
		tpl.data.penalty_total = penalty_total;
		tpl.data.penalty_count = penalty_count;
		
		if tpl.data.time_ms ~= nil then
			tpl.data.time_penalty_ms = tpl.data.time_ms + 1000*penalty_total;
			if device.led_running_time == 100 then
				-- 1/10 
				tpl.data.time_penalty = GetStringTime(tpl.data.time_penalty_ms, '%xs.%1f', '%-1h%-1m%2s.%1f', 10000);
			else
				-- seconde
				tpl.data.time_penalty = GetStringTime(tpl.data.time_penalty_ms, '%xs.0', '%-1h%-1m%2s.0', 10000);
			end
		end
	end
end

function LedDisplay_Finish(bib)
	
	local bibRanking = GetBibLoad(bib);
	if bibRanking == nil then return end
	
	local tpl = nil;
	if type(ledTemplateFinish) == 'table' then
		tpl = ledTemplateFinish;
	elseif type(ledTemplateFinish) == 'function' then
		tpl = ledTemplateFinish(bibRanking);
	end
	if tpl == nil then return end
	
	-- Effacement du Display
	SendLedCommands('-c');
	
	-- Chargement des data 
	tpl.data = {};
	tpl.data.bib = tostring(bib);
	tpl.data.identity = GetIdentity(bibRanking, 0);
	
	-- time_chrono
	local time_chrono_ms = bibRanking:GetCellInt(GetColumnTimeChrono(bibRanking), 0);
	tpl.data.time_chrono = GetStringTime(time_chrono_ms, "%xs.%2f", "%-1h%-1m%2s.%2f", 10000);
	tpl.data.time_chrono_ms = time_chrono_ms;
	
--	app.GetAuiMessage():AddLine('TC='..tpl.data.time_chrono);
	
	-- rk_chrono
	local rk_chrono = bibRanking:GetCell(GetColumnRkChrono(bibRanking), 0);
	tpl.data.rk_chrono = rk_chrono;

	-- rk_scratch_chrono 
	local rk_scratch_chrono = bibRanking:GetCell(GetColumnRkScratchChrono(bibRanking), 0);
	tpl.data.rk_scratch_chrono = rk_scratch_chrono;

	-- diff_chrono
	if device.finish_data.diff_chrono ~= nil then
		tpl.data.diff_chrono = device.finish_data.diff_chrono;
		tpl.data.diff_chrono_ms = tonumber(device.finish_data.diff_chrono);
		if string.len(device.finish_data.diff_chrono) > 0 then
			tpl.data.diff_chrono = app.TimeToString(tpl.data.diff_chrono_ms, '[DIFF]%xs.%2f');
		end
	end

	-- diff_scratch_chrono
	if device.finish_data.diff_scratch_chrono ~= nil then
		tpl.data.diff_scratch_chrono = device.finish_data.diff_scratch_chrono;
		tpl.data.diff_scratch_chrono_ms = tonumber(device.finish_data.diff_scratch_chrono);
		if string.len(device.finish_data.diff_scratch_chrono) > 0 then
			tpl.data.diff_scratch_chrono = app.TimeToString(tpl.data.diff_scratch_chrono_ms, '[DIFF]%xs.%2f');
		end
	end
	
	-- time
	local time_ms = bibRanking:GetCellInt(GetColumnTime(bibRanking), 0);
	tpl.data.time = GetStringTime(time_ms, "%xs.%2f", "%-1h%-1m%2s.%2f", 10000);
	tpl.data.time_ms = time_ms;

	-- rk
	local rk = bibRanking:GetCell(GetColumnRk(bibRanking), 0);
	tpl.data.rk = rk;

	-- rk_scratch
	local rk_scratch = bibRanking:GetCell(GetColumnRkScratch(bibRanking), 0);
	tpl.data.rk_scratch = rk_scratch;

	-- diff
	if device.finish_data.diff ~= nil then
		tpl.data.diff = device.finish_data.diff;
		tpl.data.diff_ms = tonumber(device.finish_data.diff);
		if string.len(device.finish_data.diff) > 0 then
			tpl.data.diff = app.TimeToString(tpl.data.diff_ms, '[DIFF]%xs.%2f');
		end
	end

	-- diff_scratch
	if device.finish_data.diff_scratch ~= nil then
		tpl.data.diff_scratch = device.finish_data.diff_scratch;
		tpl.data.diff_scratch_ms = tonumber(device.finish_data.diff_scratch);
		if string.len(device.finish_data.diff_scratch) > 0 then
			tpl.data.diff_scratch = app.TimeToString(tpl.data.diff_scratch_ms, '[DIFF]%xs.%2f');
		end
	end

	-- medal
	if device.entite == 'ESF' then
		tpl.data.medal = bibRanking:GetCell('Medaille'..device.key_current, 0);
	end
	
	-- time_total
	local col_time_total = GetColumnTimeTotal(bibRanking);
	if col_time_total >= 0 then
		local time_total_ms = bibRanking:GetCellInt(col_time_total, 0);
		tpl.data.time_total = GetStringTime(time_total_ms, "%xs.%2f", "%-1h%-1m%2s.%2f", 10000);
		tpl.data.time_total_ms = time_total_ms;

		-- rk_total
		local rk_total = bibRanking:GetCell(GetColumnRkTotal(bibRanking), 0);
		tpl.data.rk_total = rk_total;

		-- rk_scratch_total
		local rk_scratch_total = bibRanking:GetCell(GetColumnRkScratchTotal(bibRanking), 0);
		tpl.data.rk_scratch_total = rk_scratch_total;
	end

	-- penalty
	SetTemplateDataPenalty(tpl, bibRanking);

	-- Affichage final 
	SynchroTemplate(tpl);
end

function LedDisplay_Inter(bib, inter_passage)
	local bibRanking = GetBibLoad(bib);
	if bibRanking == nil then return end
	
	local tpl = nil;
	if type(ledTemplateInter) == 'table' then
		tpl = ledTemplateInter;
	elseif type(ledTemplateInter) == 'function' then
		tpl = ledTemplateInter(bibRanking);
	end
	if tpl == nil then return end

	-- Effacement du Display
	SendLedCommands('-c');
	
	-- Chargement des data 
	tpl.data = {};
	tpl.data.bib = tostring(bib);
	tpl.data.identity = GetIdentity(bibRanking, 0);
	tpl.data.inter = inter_passage;
	
	-- time_chrono
	local time_chrono_ms = device.inter_data.time_chrono;
	tpl.data.time_chrono = GetStringTime(time_chrono_ms, "%xs.%2f", "%-1h%-1m%2s.%2f", 10000);
	tpl.data.time_chrono_ms = time_chrono_ms;

	-- diff_chrono
	if device.inter_data.diff_chrono ~= nil then
		tpl.data.diff_chrono = device.inter_data.diff_chrono;
		tpl.data.diff_chrono_ms = tonumber(device.inter_data.diff_chrono);
		if string.len(device.inter_data.diff_chrono) > 0 then
			tpl.data.diff_chrono = app.TimeToString(tpl.data.diff_chrono_ms, '[DIFF]%xs.%2f');
		end
	end

	-- diff_scratch_chrono
	if device.inter_data.diff_scratch_chrono ~= nil then
		tpl.data.diff_scratch_chrono = device.inter_data.diff_scratch_chrono;
		tpl.data.diff_scratch_chrono_ms = tonumber(device.inter_data.diff_scratch_chrono);
		if string.len(device.inter_data.diff_scratch_chrono) > 0 then
			tpl.data.diff_scratch_chrono = app.TimeToString(tpl.data.diff_scratch_chrono_ms, '[DIFF]%xs.%2f');
		end
	end
	
	-- penalty
	SetTemplateDataPenalty(tpl, bibRanking);

	-- Affichage final 
	SynchroTemplate(tpl);

end

function LedDisplay_Running()
	if tRunning.bib ~= '' and type(tRunning.bibLoad) == 'userdata' and tRunning.bibLoad:GetCell('Dossard', 0) == tRunning.bib then

		local tpl = nil;
		if type(ledTemplateRunning) == 'table' then
			tpl = ledTemplateRunning;
		elseif type(ledTemplateRunning) == 'function' then
			tpl = ledTemplateRunning(tRunning.bibLoad);
		end
		if tpl == nil then return end

		if device.led_running_time > 0 then
			-- Temps Tournant au 1/10 ou  la seconde ...
	
			tpl.data = {};
			tpl.data.bib = tostring(tRunning.bib);
			tpl.data.identity = GetIdentity(tRunning.bibLoad, 0);
			
			if device.led_running_time == 100 then
				-- 1/10 
				tpl.data.time = GetStringTime(tRunning.time, '%xs.%1f', '%-1h%-1m%2s.%1f', 10000);
			else
				-- seconde
				tpl.data.time = GetStringTime(tRunning.time, '%xs.0', '%-1h%-1m%2s.0', 10000);
			end
			tpl.data.time_ms = tRunning.time;
			
			SetTemplateDataPenalty(tpl, tRunning.bibLoad);

			SynchroTemplate(tpl);
		end
	end
end

function LedDisplay_Message()
	if device.led_node ~= nil then
		local msg1 = device.led_node:GetAttribute("msg1_text");
		local optimize = 1;
		local clear = 0;
		if string.len(msg1) > 0 then
			local color1 = device.led_node:GetAttribute('msg1_color', 'red');
			local timer1 = tonumber(device.led_node:GetAttribute('msg1_timer', '0')) or 0;
			if timer1 > 0 then
				msg1 = msg1..'          ';
				SendLedCommands('-scroll8x8 "'..msg1..'" '..color1..' 0 0 '..tostring(device.led_col)..' 8 '..tostring(timer1));
			else
				SendLedCommands('-t8x8 "'..msg1..'" '..color1..' 0 0 '..optimize..' '..clear);
			end
		end
	
		local msg2 = device.led_node:GetAttribute("msg2_text");
		if string.len(msg2) > 0 then
			msg2 = msg2..'          ';
			local color2 = device.led_node:GetAttribute('msg2_color', 'red');
			local timer2 = tonumber(device.led_node:GetAttribute('msg2_timer', '0')) or 0;
			if timer2 > 0 then
				SendLedCommands('-scroll8x8 "'..msg2..'" '..color2..' 0 8 '..tostring(device.led_col)..' 8 '..tostring(timer2));
			else
				SendLedCommands('-t8x8 "'..msg2..'" '..color2..' 0 8 '..optimize..' '..clear);
			end
		end
		
		SendLedCommands('-s');
	end
end

function LedDisplay_Test_Clock()
	device.led_mode = 'message';
	SendLedCommands('-c all -s');
--[[
	SendLedCommands('-clock color before_warning yellow');
	SendLedCommands('-clock color before_critical yellow');
	SendLedCommands('-clock color after green');
	SendLedCommands('-clock color default red');
	
	SendLedCommands('-clock sound go 600f50_700f50_800f50_900f50_1000f50_900f50_800f50_700f50 15000');
]]	

	SendLedCommands('-clock synchro 100000');
	SendLedCommands('-clock first_start 100030');
	SendLedCommands('-clock start 30 10 5 5');	-- Period 30, Warning 10, Critical 5, After 5
	
--	SendLedCommands('-clock synchro 120000');
end

function LedDisplay_Test_Sound()
	device.led_mode = 'message';
	SendLedCommands('-c all -s');

	local soundValue = '';
	for i=1,10 do
		if soundValue:len() > 0 then
			soundValue = soundValue..'_';
		end
		
		soundValue = soundValue..tostring(500+i*50)..'f100';
	end
	for i=9,1,-1 do
		soundValue = soundValue..'_';
		soundValue = soundValue..tostring(500+i*50)..'f100';
	end
--	SendLedCommands('--sound create test '..soundValue);
--	SendLedCommands('-sound create test 100f100_200f100_300f100_400f100_500f100_600f100_700f100_800f100');

--	SendLedCommands('--sound create test 600f150_900f150'); -- pas mal lent
--	SendLedCommands('--sound create test 600f100_900f100'); -- pas mal plus rapide
--	SendLedCommands('--sound create test 600f75_900f75'); -- pas mal encore plus rapide
--	SendLedCommands('--sound create test 600f75_900f150');

--	SendLedCommands('--sound create test 600f200_900f100'); -- pas mal aussi
--	SendLedCommands('--sound create test 500f100_1000f100'); 

--	SendLedCommands('--sound create test 1f50_0f50'); -- Beep Rapide ...
--	SendLedCommands('--sound create test 1f100_0f100'); -- Beep Lent ...
--	SendLedCommands('--sound create test 1f300_0f300'); -- Beep Tres Lent ...
--	SendLedCommands('--sound create test 1f50_0f200'); -- Autre Beep  ...
--	SendLedCommands('--sound create test 600f100_900f300'); -- Son Alarme

--	SendLedCommands('-sound create test 1f50_0f50')
	SendLedCommands('-sound create test 1f1000')
	SendLedCommands('-sound play test 1000');
end

function LedDisplay_Test_Finish()
	device.led_mode = 'message';
	SendLedCommands('-c all -s');

	local bibRanking = GetBibLoad(1);

	local tpl = nil;
	if type(ledTemplateFinish) == 'table' then
		tpl = ledTemplateFinish;
	elseif type(ledTemplateFinish) == 'function' then
		tpl = ledTemplateFinish(bibRanking);
	end
	if tpl == nil then return end

	-- Chargement des data 
	tpl.data = {};
	tpl.data.bib = '251';
	tpl.data.identity = 'COSSERAT Pierre';

	-- time_chrono
	local time_chrono_ms = 11*60000+23*1000+220;
	tpl.data.time_chrono = GetStringTime(time_chrono_ms, "%xs.%2f", "%-1h%-1m%2s.%2f", 10000);
	tpl.data.time_chrono_ms = time_chrono_ms;
	
	-- rk_chrono
	local rk_chrono = 98;
	tpl.data.rk_chrono = rk_chrono;

	-- rk_scratch_chrono 
	local rk_scratch_chrono = 99;
	tpl.data.rk_scratch_chrono = rk_scratch_chrono;

	-- diff_chrono
	tpl.data.diff_chrono = '+1.13';
	tpl.data.diff_chrono_ms = 1013;

	-- diff_scratch_chrono
	tpl.data.diff_scratch_chrono = '+1.12';
	tpl.data.diff_scratch_chrono = 1012;
	
	-- time
	local time_ms = 11*60000+23*1000+220;
	tpl.data.time = GetStringTime(time_ms, "%xs.%2f", "%-1h%-1m%2s.%2f", 10000);
	tpl.data.time_ms = time_ms;

	-- rk
	local rk = 99;
	tpl.data.rk = rk;

	-- rk_scratch
	local rk_scratch = 99;
	tpl.data.rk_scratch = rk_scratch;

	-- diff
	tpl.data.diff = '+1.12';
	tpl.data.diff_ms = 1012;

	-- diff_scratch
	tpl.data.diff_scratch = '+1.12';
	tpl.data.diff_scratch_ms = 1012;
	
	-- penalty
	SetTemplateDataPenalty(tpl, bibRanking);

	-- Affichage final 
	SynchroTemplate(tpl);
end
